'use strict';

const jsonRead = require('../utils/json-read');
const requestWrapper = require('./request-wrapper');
const fs = require('fs');
const querystring = require('querystring');

/*
 * List of Alexa Skill Management API Service Calls.
 * These functions are thin wrappers for Alexa Skill Management APIs.
 */

module.exports = {
    // simulation
    callSimulateSkill: callSimulateSkill,
    callGetSimulation: callGetSimulation,
    callInvokeSkill: callInvokeSkill,
    // skill
    callCreateSkill: callCreateSkill,
    callGetSkill: callGetSkill,
    handleGetSkillRedirect: handleGetSkillRedirect,
    callUpdateSkill: callUpdateSkill,
    callDeleteSkill: callDeleteSkill,
    callGetSkillStatus: callGetSkillStatus,
    callListSkills: callListSkills,
    // model
    callGetModel: callGetModel,
    callHeadModel: callHeadModel,
    callUpdateModel: callUpdateModel,
    // account linking
    callCreateAccountLinking: callCreateAccountLinking,
    callGetAccountLinking: callGetAccountLinking,
    callDeleteAccountLinking: callDeleteAccountLinking,
    // misc
    callListVendor: callListVendor,
    callSubmit: callSubmit,
    callWithdraw: callWithdraw,
    callEnableSkill: callEnableSkill,
    callDisableSkill: callDisableSkill,
    callGetSkillEnablement: callGetSkillEnablement,
    // private skill
    callAddPrivateDistributionAccount: callAddPrivateDistributionAccount,
    callDeletePrivateDistributionAccount: callDeletePrivateDistributionAccount,
    callListPrivateDistributionAccounts: callListPrivateDistributionAccounts
};


function callInvokeSkill(file, jsonObject, skillId, endpointRegion, profile, doDebug, callback) {
    let requestPayload = file ? jsonRead.readFile(file) : jsonObject;
    let invokeRequestPayload = {
        "endpointRegion": endpointRegion,
        "skillRequest": {
            "body": requestPayload
        }
    };

    let general = {
        url: '/skills/' + skillId + '/invocations',
        method: 'POST'
    };
    let headers = {};
    requestWrapper.request('invoke-skill', general, headers, invokeRequestPayload, profile, doDebug, callback);
}

function callSimulateSkill(file, text, skillId, locale, profile, doDebug, callback) {
    let payload = {
        "input": {
            "content": file ? fs.readFileSync(file, 'utf-8') : text
        },
        "device": {
            "locale": locale
        }
    };

    let general = {
        url: '/skills/' + skillId + '/simulations',
        method: 'POST'
    };
    let headers = {};
    requestWrapper.request('simulate-skill', general, headers, payload, profile, doDebug, callback);
}

function callGetSimulation(simulationId, skillId, profile, doDebug, callback) {
    let general = {
        url: '/skills/' + skillId + '/simulations/' + simulationId,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-simulation', general, headers, null, profile, doDebug, callback);
}

function callCreateSkill(manifest, vendorId, profile, doDebug, callback) {
    let general = {
        url: '/skills/',
        method: 'POST'
    };
    let headers = {};
    let payload = {
        vendorId: vendorId,
        manifest: manifest.manifest
    };
    requestWrapper.request('create-skill', general, headers, payload, profile, doDebug, callback);
}

function callGetSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/manifest';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill', general, headers, null, profile, doDebug, callback);
}

function handleGetSkillRedirect(redirectUrl, profile, doDebug, callback) {
    let headers = {};
    requestWrapper.requestWithUrl('get-skill-status', redirectUrl, 'GET', headers, null, profile, doDebug, callback);
}

function callUpdateSkill(skillId, stage, manifest, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/manifest';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    let payload = {
        manifest: manifest.manifest
    };
    requestWrapper.request('update-skill', general, headers, payload, profile, doDebug, callback);
}

function callDeleteSkill(skillId, profile, doDebug, callback) {
    let url = '/skills/' + skillId;
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-skill', general, headers, null, profile, doDebug, callback);
}

function callGetSkillStatus(skillId, resourcesList, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/status';
    if (resourcesList !== null && resourcesList.length !== 0) {
        url += ('?' + querystring.stringify({ resource: resourcesList }));
    }

    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill-status', general, headers, null, profile, doDebug, callback);
}

function callListSkills(vendorId, nextToken, numberOfReturnSkills, profile, doDebug, callback) {
    let url = nextToken ?
        '/skills?vendorId=' + vendorId + '&nextToken=' + nextToken + '&maxResults=' + (numberOfReturnSkills || 50) :
        '/skills?vendorId=' + vendorId + '&maxResults=' + (numberOfReturnSkills || 50);

    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-skills', general, headers, null, profile, doDebug, callback);
}

function callGetModel(skillId, stage, locale, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-model', general, headers, null, profile, doDebug, callback);
}

function callHeadModel(skillId, stage, locale, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'HEAD'
    };
    let headers = {};
    requestWrapper.request('head-model', general, headers, null, profile, doDebug, callback);
}

function callUpdateModel(skillId, stage, locale, modelSchema, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/interactionModel/locales/' + locale;
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    let payload = {
        interactionModel: modelSchema.interactionModel
    };
    requestWrapper.request('update-model', general, headers, payload, profile, doDebug, callback);
}

function callCreateAccountLinking(skillId, stage, accountLinking, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    let payload = {
        accountLinkingRequest: accountLinking
    };
    requestWrapper.request('create-account-linking', general, headers, payload, profile, doDebug, callback);
}

function callGetAccountLinking(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-account-linking', general, headers, null, profile, doDebug, callback);
}

function callDeleteAccountLinking(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/accountLinkingClient';
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-account-linking', general, headers, null, profile, doDebug, callback);
}

function callListVendor(profile, doDebug, callback) {
    let url = '/vendors';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-vendors', general, headers, null, profile, doDebug, callback);
}

function callSubmit(skillId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/submit';
    let general = {
        url: url,
        method: 'POST'
    };
    let headers = {};
    requestWrapper.request('submit', general, headers, null, profile, doDebug, callback);
}

function callWithdraw(skillId, withdrawReason, withdrawMessage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/withdraw';
    let general = {
        url: url,
        method: 'POST'
    };
    let headers = {};
    let payload = {
        reason: withdrawReason,
        message: withdrawMessage
    };
    requestWrapper.request('withdraw', general, headers, payload, profile, doDebug, callback);
}

function callEnableSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    requestWrapper.request('enable-skill', general, headers, null, profile, doDebug, callback);
}

function callDisableSkill(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('disable-skill', general, headers, null, profile, doDebug, callback);
}

function callGetSkillEnablement(skillId, stage, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/enablement';
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('get-skill-enablement', general, headers, null, profile, doDebug, callback);
}

function callAddPrivateDistributionAccount(skillId, stage, accountId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts/' + accountId;
    let general = {
        url: url,
        method: 'PUT'
    };
    let headers = {};
    requestWrapper.request('add-private-distribution-account', general, headers, null, profile, doDebug, callback);
}

function callDeletePrivateDistributionAccount(skillId, stage, accountId, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts/' + accountId;
    let general = {
        url: url,
        method: 'DELETE'
    };
    let headers = {};
    requestWrapper.request('delete-private-distribution-account', general, headers, null, profile, doDebug, callback);
}

function callListPrivateDistributionAccounts(skillId, stage, nextToken, numberOfReturnedAccounts, profile, doDebug, callback) {
    let url = '/skills/' + skillId + '/stages/' + stage + '/privateDistributionAccounts?';
    url += nextToken ? 
        'nextToken=' + nextToken + '&maxResults=' + (numberOfReturnedAccounts || 50) :
        'maxResults=' + (numberOfReturnedAccounts || 50);
    let general = {
        url: url,
        method: 'GET'
    };
    let headers = {};
    requestWrapper.request('list-private-distribution-accounts', general, headers, null, profile, doDebug, callback);
}
