'use strict';

const expect = require('chai').expect;
const sinon = require('sinon');
const jsonRead = require('../../lib/utils/json-read');
const oauthWrapper = require('../../lib/utils/oauth-wrapper');
const fs = require('fs');
const jsonUtility = require('../../lib/utils/json-utility');
const CONSTANTS = require('../../lib/utils/constants');
const apiWrapper = require('../../lib/api/api-wrapper');

describe('api api-wrapper testing', () => {
    let sandbox;
    let emptyCallback = () => {};
    const TEST_PROFILE = 'test';
    const VENDOR_ID = 'vendorId';

    beforeEach(() => {
        sandbox = sinon.sandbox.create();

        sandbox.stub(jsonRead, 'readFile');
        sandbox.stub(jsonRead, 'getProperty');
        sandbox.stub(fs, 'existsSync');
        sandbox.stub(oauthWrapper, 'tokenRefreshAndRead');
        sandbox.stub(jsonUtility, 'getProperty');
    });

    afterEach(() => {
        sandbox.restore();
    });

    describe('# create-skill', () => {
        it ('| submit create-skill request correctly', ()=> {
            const SKILL_MANIFEST = {
                manifest: 'content of skill manifest'
            };
            
            apiWrapper.callCreateSkill(SKILL_MANIFEST, VENDOR_ID, TEST_PROFILE, false, emptyCallback);

            expect(oauthWrapper.tokenRefreshAndRead.called).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/')).equal(true);
            expect(param.body.manifest).equal('content of skill manifest');
            expect(param.body.vendorId).equal(VENDOR_ID);
        });
    });

    describe('# get-skill', () => {
        it ('| will finally make a request to get skill', () => {
            apiWrapper.callGetSkill('skill-id', 'stage', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/manifest')).equal(true);
        });
    });

    describe('# handle-get-skill-redirect', () => {
        it ('| will finally make a request to get skill', () => {
            apiWrapper.handleGetSkillRedirect('url', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('url')).equal(true);
        });
    });

    describe('# update-skill', () => {
        it ('| will finally make a request to update skill', () => {
            apiWrapper.callUpdateSkill('skill-id', 'stage', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/manifest')).equal(true);
        });
    });

    describe('# delete-skill', () => {
        it ('| will finally make a request to delete skill', () => {
            apiWrapper.callDeleteSkill('skill-id', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id')).equal(true);
        });
    });
    
    describe('# get-model', () => {
        it ('| will finally make a request to get model', () => {
            apiWrapper.callGetModel('skill-id', 'stage', 'locale', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/interactionModel/locales/locale')).equal(true);
            
        });
    });

    describe('# head-model', () => {
        it ('| will finally make a request to head model', () => {
            apiWrapper.callHeadModel('skill-id', 'stage', 'locale', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/interactionModel/locales/locale')).equal(true);
        });
    });

    describe('# update-model', () => {
        it ('| will finally make a request to update model', () => {
            const MODEL_SCHEMA = {
                interactionModel: 'model content'
            };
            jsonRead.readFile.returns(true);

            apiWrapper.callUpdateModel('skill-id', 'stage', 'locale', MODEL_SCHEMA, TEST_PROFILE, false, emptyCallback);

            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/interactionModel/locales/locale')).equal(true);
            expect(param.body.interactionModel).equal('model content');
        });
    });
    
    describe('# create-account-linking', () => {
        it ('| will finally make a request to create account linking', () => {
            apiWrapper.callCreateAccountLinking('skill-id', 'stage', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/accountLinkingClient')).equal(true);
        });
    });

    describe('# get-account-linking', () => {
        it ('| will finally make a request to get account linking', () => {
            apiWrapper.callGetAccountLinking('skill-id', 'stage', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/accountLinkingClient')).equal(true);
        });
    });

    describe('# delete-account-linking', () => {
        it ('| will finally make a request to get account linking', () => {
            apiWrapper.callDeleteAccountLinking('skill-id', 'stage', TEST_PROFILE, false, emptyCallback);
            
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let param = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(param.url.includes('/skills/skill-id/stages/stage/accountLinkingClient')).equal(true);
        });
    });

    describe('# list-vendors', () => {
        it ('| will finally make a request to get list of vendors', () => {
            apiWrapper.callListVendor('skill-id', TEST_PROFILE, false, emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
        });
    });

    describe('# list-skills', () => {
        it ('| send list-skills request without nextToken', () => {
            // act
            apiWrapper.callListSkills(123, null, 50, TEST_PROFILE, false, emptyCallback);

            // assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('GET');
            expect(arg.url.includes('/skills?vendorId=123&maxResults=50')).equal(true);
        });

        it ('| send list-skills request with nextToken', () => {
            // act
            apiWrapper.callListSkills(123, 321, 50, TEST_PROFILE, false, emptyCallback);

            // assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('GET');
            expect(arg.url.includes('/skills?vendorId=123&nextToken=321&maxResults=50')).equal(true);
        });
    });

    describe('# submit-skill', () => {
        it ('| will send an submit-skill request', () => {
            apiWrapper.callSubmit('skill-id', TEST_PROFILE, false, emptyCallback);

            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);

            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/submit')).equal(true);
        });
    });

    describe('# withdraw-skill', () => {
        it ('| will send an withdraw-skill request correctly', () => {
            const WITHDRAW_REASON = 'reason';
            const WITHDRAW_MESSAGE = 'message';
            apiWrapper.callWithdraw('skill-id', WITHDRAW_REASON, WITHDRAW_MESSAGE, TEST_PROFILE, false, emptyCallback);

            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);

            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/withdraw')).equal(true);
            expect(arg.body.reason).equal(WITHDRAW_REASON);
            expect(arg.body.message).equal(WITHDRAW_MESSAGE);
        });
    });

    describe('# enable-skill', () => {
        it ('| will send an enablement request', () => {
            apiWrapper.callEnableSkill('skill-id', CONSTANTS.SKILL.STAGE.DEVELOPMENT, TEST_PROFILE, false, emptyCallback);

            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);

            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/stages/development/enablement')).equal(true);
        });
    });

    describe('# disable-skill', () => {
        it ('| will send a delete enablement request', () => {
            apiWrapper.callDisableSkill('skill-id', CONSTANTS.SKILL.STAGE.DEVELOPMENT, TEST_PROFILE, false, emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);

            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('DELETE');
            expect(arg.url.includes('/skills/skill-id/stages/development/enablement')).equal(true);
        });
    });

    describe('# get-skill-enablement', () => {
        it ('| will retrieve the skill enablement', () => {
            apiWrapper.callGetSkillEnablement('skill-id', CONSTANTS.SKILL.STAGE.DEVELOPMENT, TEST_PROFILE, false, emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);

            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('GET');
            expect(arg.url.includes('/skills/skill-id/stages/development/enablement')).equal(true);
        });
    });

    describe('# simulate-skill', () => {
         it ('| verify the request params are correct with file input', (done) => {
            //set up
            sandbox.stub(fs, 'readFileSync').returns('Simulation Content');

            //act
            apiWrapper.callSimulateSkill('file', null, '12345', 'en-us', TEST_PROFILE, false, done);

            //assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('POST');
            expect(arg.url.includes('/skills/12345/simulations')).equal(true);
            expect(arg.json).equal(true);
            expect(arg.body.input.content).equal('Simulation Content');
            expect(arg.body.device.locale).equal('en-us');
            done();
        });

        it ('| verify the request params are correct with text input', (done) => {
            //act
            apiWrapper.callSimulateSkill(null, 'Simulation Content', '12345', 'en-us', TEST_PROFILE, false, done);

            //assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('POST');
            expect(arg.url.includes('/skills/12345/simulations')).equal(true);
            expect(arg.json).equal(true);
            expect(arg.body.input.content).equal('Simulation Content');
            expect(arg.body.device.locale).equal('en-us');
            done();
        });
    });

    describe('# get-simulation', () => {
        it ('| verify the request params are correct', (done) => {
            //set up
            jsonRead.readFile.returns(true);
            fs.existsSync.returns(true);
            jsonRead.getProperty.returns('vendorId');

            //act
            apiWrapper.callGetSimulation('12345', '67890', done);

            //assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('GET');
            expect(arg.url.includes('/skills/67890/simulations/12345')).equal(true);
            expect(arg.json).equal(false);
            expect(arg.body).equal(null);
            done();
        });
    });

    describe('# add-private-distribution-account', () => {
        it ('| will finally make a request to add private distribution account', () => {
          apiWrapper.callAddPrivateDistributionAccount('skill-id', 'stage', 'account-id', emptyCallback);
          expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
        });
    });

    describe('# delete-private-distribution-account', () => {
        it ('| will finally make a request to delete private distribution account', () => {
          apiWrapper.callDeletePrivateDistributionAccount('skill-id', 'stage', 'account-id', emptyCallback);
          expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
        });
    });

    describe('# list-private-distribution-accounts', () => {
        it ('| will finally make a request to list private distribution accounts', () => {
          apiWrapper.callListPrivateDistributionAccounts('skill-id', 'stage', null, 50, emptyCallback);
          expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
        });

        it ('| will finally make a request to list private distribution accounts with nextToken', () => {
          apiWrapper.callListPrivateDistributionAccounts('skill-id', 'stage', 'nextToken', 50, emptyCallback);
          expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
        });
    });

    describe('# invoke-skill', () => {
        it ('| verify the request params are correct with file input', (done) => {
            let requestBody = {
              "session": {
                "sessionId": "SessionId.e12ea327-6752-4679-bf85-c65692a91c2b",
                "application": {
                  "applicationId": "amzn1.ask.skill.0c24c87b-434d-43d8-a227-8c7e52a4e819"
                },
                "attributes": {},
                "user": {
                  "userId": "amzn1.ask.account.AEZEGAFXFANE7AAAAAAZFLIBIBNJBL3LWNZHFSW2FO4ESH6YRPHZPQ6DNJ5QI6FPILBWR6UOFHYLCMLJM7K7O5WGTAUM5BGLXUJ7CRICV6B2Q6RH3WXZOBPZVQBUI4YUOWOMJCD3ULJ3J6D4C43OIJDQFANVCXKIPQH4X4CY6XGFQS4C5ADQSIHOQKPKJOU4KHE5ZF32LPI74NQ"
                },
                "new": true
              },
              "request": {
                "type": "IntentRequest",
                "requestId": "EdwRequestId.ba7aa96c-4ac2-4d6a-85f4-cf8a1f129ffd",
                "locale": "en-US",
                "timestamp": "2017-06-15T03:46:55Z",
                "intent": {
                  "name": "RecipeIntent",
                  "slots": {
                    "Item": {
                      "name": "Item",
                      "value": "map"
                    }
                  }
                }
              },
              "version": "1.0"
            };

            //set up
            jsonRead.readFile.returns(requestBody);
            fs.existsSync.returns(true);
            jsonRead.getProperty.returns('vendorId');

            //act
            apiWrapper.callInvokeSkill("file", null, '12345', 'NA', TEST_PROFILE, done);

            //assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('POST');
            expect(arg.url.includes('/skills/12345/invocations')).equal(true);
            expect(arg.json).equal(true);
            expect(arg.body.endpointRegion).equal('NA');
            expect(JSON.stringify(arg.body.skillRequest.body.request) === JSON.stringify(requestBody.request)).equal(true);
            done();
        });

        it ('| verify the request params are correct with json object input', (done) => {
            let requestBody = {
              "session": {
                "sessionId": "SessionId.e12ea327-6752-4679-bf85-c65692a91c2b",
                "application": {
                  "applicationId": "amzn1.ask.skill.0224c87b-437d-43d8-a227-8c7e52a4e819"
                },
                "attributes": {},
                "user": {
                  "userId": "amzn1.ask.account.AEZEGA6XAANE7TZGKWAZFAAAAAAABL3LWNZHFSW2FO4ESH6YRPHZPQ6DNJ5QI6FPILBWR6UOFHYLCMLJM7K7O5WGTAUM5BGLXUJ7CRICV6B2Q6RH3WXZOBPZVQBUI4YUOWOMJCD3ULJ3J6D4C43OIJDQFANVCXKIPQH4X4CY6XGFQS4C5ADQSIHOQKPKJOU4KHE5ZF32LPI74NQ"
                },
                "new": false
              },
              "request": {
                "type": "IntentRequest",
                "requestId": "EdwRequestId.ba7aa96c-4ac2-4d6a-85f4-cf8a1f129ffd",
                "locale": "en-US",
                "timestamp": "2017-06-15T03:46:55Z",
                "intent": {
                  "name": "RecipeIntent",
                  "slots": {
                    "Item": {
                      "name": "Item",
                      "value": "map"
                    }
                  }
                }
              },
              "version": "1.0"
            };

            //set up
            jsonRead.readFile.returns(true);
            fs.existsSync.returns(true);
            jsonRead.getProperty.returns('vendorId');

            //act
            apiWrapper.callInvokeSkill(null, requestBody, '12345', 'EU', TEST_PROFILE, done);

            //assert
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.method).equal('POST');
            expect(arg.url.includes('/skills/12345/invocations')).equal(true);
            expect(arg.json).equal(true);
            expect(arg.body.endpointRegion).equal('EU');
            expect(JSON.stringify(arg.body.skillRequest.body.request) === JSON.stringify(requestBody.request)).equal(true);
            done();
        });
    });
    describe('# get-skill-status', () => {
        it ('| make a request to get skill status for both by default with null list', () => {
            apiWrapper.callGetSkillStatus('skill-id', null, 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status')).equal(true);
        });

        it ('| make a request to get skill status for both by default with empty list', () => {
            apiWrapper.callGetSkillStatus('skill-id', [], 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status')).equal(true);
        });

        it ('| make a request to get skill status for both by explicitly setting the flags', () => {
            apiWrapper.callGetSkillStatus('skill-id', [CONSTANTS.SKILL.RESOURCES.MANIFEST, CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL], 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status?resource=manifest&resource=interactionModel')).equal(true);
        });

        it ('| make a request to get skill status for manifest', () => {
            apiWrapper.callGetSkillStatus('skill-id', [CONSTANTS.SKILL.RESOURCES.MANIFEST], 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status?resource=manifest')).equal(true);
        });

        it ('| make a request to get skill status for model', () => {
            apiWrapper.callGetSkillStatus('skill-id', [CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL], 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status?resource=interactionModel')).equal(true);
        });

        it ('| make a request to get skill status for model with multiple duplicate item in the list', () => {
            let resourceList = [CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL, 
                                CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL, 
                                CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL,
                                CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL, 
                                CONSTANTS.SKILL.RESOURCES.INTERACTION_MODEL];
            apiWrapper.callGetSkillStatus('skill-id', resourceList, 'default', emptyCallback);
            expect(oauthWrapper.tokenRefreshAndRead.calledOnce).equal(true);
            let arg = oauthWrapper.tokenRefreshAndRead.getCall(0).args[0];
            expect(arg.url.includes('/skills/skill-id/status?resource=interactionModel')).equal(true);
        });
    });
});
